"use strict";
/*!
* Contentstack DataSync Manager
* Copyright (c) 2019 Contentstack LLC
* MIT Licensed
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.debugNotifications = exports.start = exports.notifications = exports.setLogger = exports.getConfig = exports.setConfig = exports.setListener = exports.setAssetStore = exports.setContentStore = exports.getAssetLocation = exports.pop = exports.unshift = exports.push = void 0;
const debug_1 = __importDefault(require("debug"));
const lodash_1 = require("lodash");
const config_1 = require("./config");
const index_1 = require("./core/index");
const inet_1 = require("./core/inet");
const process_1 = require("./core/process");
const q_1 = require("./core/q");
Object.defineProperty(exports, "notifications", { enumerable: true, get: function () { return q_1.notifications; } });
const build_paths_1 = require("./util/build-paths");
const index_2 = require("./util/index");
const logger_1 = require("./util/logger");
const validations_1 = require("./util/validations");
const debug = (0, debug_1.default)('sm:index');
let assetStoreInstance;
let appConfig = {};
let contentStore;
let assetStore;
let listener;
const push = (data) => {
    (0, validations_1.validateExternalInput)(data);
    (0, index_1.push)(data);
};
exports.push = push;
const unshift = (data) => {
    (0, validations_1.validateExternalInput)(data);
    (0, index_1.unshift)(data);
};
exports.unshift = unshift;
const pop = () => {
    (0, index_1.pop)();
};
exports.pop = pop;
const getAssetLocation = (asset) => {
    return new Promise((resolve, reject) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const assetStoreConfig = assetStore.getConfig();
            const assetConfig = (assetStoreConfig.assetStore) ? assetStoreConfig.assetStore : assetStoreConfig;
            const location = yield assetStore.getAssetLocation(asset, assetConfig);
            return resolve(location);
        }
        catch (error) {
            return reject(error);
        }
    }));
};
exports.getAssetLocation = getAssetLocation;
/**
 * @public
 * @method setContentStore
 * @summary Register content store
 * @param {object} instance Content store instance
 */
const setContentStore = (instance) => {
    contentStore = instance;
};
exports.setContentStore = setContentStore;
/**
 * @public
 * @method setAssetStore
 * @summary Register asset store
 * @param {object} instance Asset store instance
 */
const setAssetStore = (instance) => {
    assetStore = instance;
};
exports.setAssetStore = setAssetStore;
/**
 * @public
 * @method setListener
 * @summary Register listener
 * @param {object} instance Listener instance
 */
const setListener = (instance) => {
    (0, validations_1.validateListener)(instance);
    listener = instance;
};
exports.setListener = setListener;
/**
 * @public
 * @method setConfig
 * @summary Sets the application's configuration
 * @param {object} config Application config
 */
const setConfig = (config) => {
    appConfig = config;
};
exports.setConfig = setConfig;
/**
 * @public
 * @method getConfig
 * @summary Returns the application's configuration
 * @returns {object} Application config
 */
const getConfig = () => {
    return appConfig;
};
exports.getConfig = getConfig;
/**
 * @public
 * @method setLogger
 * @summary Sets custom logger for logging data sync operations
 * @param {object} instance Custom logger instance
 */
var logger_2 = require("./util/logger");
Object.defineProperty(exports, "setLogger", { enumerable: true, get: function () { return logger_2.setLogger; } });
/**
 * @public
 * @method start
 * @summary Starts the sync manager utility
 * @description
 * Registers, validates asset, content stores and listener instances.
 * Once done, builds the app's config and logger
 * @param {object} config Optional application config overrides
 * @returns {Promise} Returns a promise
 */
const start = (config = {}) => {
    return new Promise((resolve, reject) => {
        try {
            (0, validations_1.validateAssetStore)(assetStore);
            (0, validations_1.validateContentStore)(contentStore);
            (0, validations_1.validateListener)(listener);
            appConfig = (0, lodash_1.merge)({}, config_1.config, appConfig, config);
            (0, validations_1.validateConfig)(appConfig);
            appConfig.paths = (0, build_paths_1.buildConfigPaths)();
            // since logger is singleton, if previously set, it'll return that isnstance!
            (0, logger_1.setLogger)();
            (0, process_1.configure)();
            return assetStore.start(appConfig).then((assetInstance) => {
                debug('Asset store instance has returned successfully!');
                (0, validations_1.validateAssetStoreInstance)(assetInstance);
                assetStoreInstance = assetInstance;
                return contentStore.start(assetInstance, appConfig);
            }).then((contentStoreInstance) => {
                debug('Content store instance has returned successfully!');
                (0, validations_1.validateContentStoreInstance)(contentStoreInstance);
                appConfig = (0, index_2.formatSyncFilters)(appConfig);
                return (0, index_1.init)(contentStoreInstance, assetStoreInstance);
            }).then(() => {
                debug('Sync Manager initiated successfully!');
                listener.register(index_1.poke);
                // start checking for inet 10 secs after the app has started
                (0, inet_1.init)();
                return listener.start(appConfig);
            }).then(() => {
                logger_1.logger.info('Contentstack sync utility started successfully!');
                return resolve('');
            }).catch(reject);
        }
        catch (error) {
            return reject(error);
        }
    });
};
exports.start = start;
/**
 * @public
 * @method debugNotifications
 * @param {object} item Item being processed
 * @returns {void}
 */
const debugNotifications = (action) => {
    return (item) => {
        debug(`Notifications: ${action} - ${JSON.stringify(item)}`);
    };
};
exports.debugNotifications = debugNotifications;
q_1.notifications
    .on('publish', (0, exports.debugNotifications)('publish'))
    .on('unpublish', (0, exports.debugNotifications)('unpublish'))
    .on('delete', (0, exports.debugNotifications)('delete'))
    .on('error', (0, exports.debugNotifications)('error'));
